#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>

#include "arch.h"
#include "extr.h"
#include "fs.h"
#include "link.h"
#include "msg.h"
#include "opts.h"
#include "path.h"
#include "str.h"



struct extractor *Extractors;



void match_extr(struct extractor *extr, const char *file) {
	regmatch_t pos;

	if (
		regexec(&extr->match, file, 1, &pos, 0) == 0 &&
		pos.rm_so > 1 &&
		file[pos.rm_so - 1] == '.'
	)
		extr->pos = pos.rm_so;
	else
		extr->pos = 0;
}


struct extractor *find_extr(struct archive *arch) {
	struct extractor *found = NULL, *extr;
	regoff_t leftmost = 0;

	for (extr = Extractors; extr; extr = extr->next) {
		match_extr(extr, arch->links.file.e);
		if (extr->pos > 0 && (!found || extr->pos < leftmost))
			if (
				!Extension ||
				strcasecmp(Extension, arch->links.file.e + extr->pos) == 0
			) {
				found = extr;
				leftmost = extr->pos;
				if (Extension)
					break;
			}
	}

	return found;
}


void add_extr_list(const char *path) {
	char *abs, *b;
	DIR *dir = opendir(path);
	struct dirent *ent;
	size_t path_len, ent_len;
	struct extractor *extr;

	if (!dir)
		return;

	path_len = strlen(path);

	while ((ent = readdir(dir))) {
		if (is_dot(ent->d_name))
			continue;

		extr = emalloc(sizeof(struct extractor));
		ent_len = strlen(ent->d_name);
		abs = b = emalloc(path_len + 1 + ent_len + sizeof "/subdirectory");
		mem_cat(&b, path, path_len);

		*b++ = '(';
		mem_cat(&b, ent->d_name, ent_len);
		strcpy(b, ")$");
		if (regcomp(&extr->match, abs + path_len, REG_EXTENDED | REG_ICASE) != 0) {
			msg(MSG_REGCOMP, NULL, abs + path_len);
			free(extr);
			free(abs);
			continue;
		}

		abs[path_len] = '/';

		strcpy(b, "/permissions");
		extr->perms = file_exist_v(abs);
		strcpy(b, "/subdirectory");
		extr->subdir = file_exist_v(abs);
		strcpy(b, "/extract");
		extr->extract = abs;

		extr->next = Extractors;
		Extractors = extr;
	}

	closedir(dir);
}


BOOL add_extrs(void) {
	char *home = getenv("HOME");

	add_extr_list(SHARE);

	/* last added extractors enjoy priority (extractor list is a FIFO) */
	if (home) {
		size_t len;
		char *local, *b;

		len = strlen(home);
		local = b = emalloc(len + sizeof("/.deco"));
		mem_cat(&b, home, len);
		strcpy(b, "/.deco");
		add_extr_list(local);
		free(local);
	}

	if (Extractors)
		return TRUE;
	msg(MSG_NO_EXTRS, NULL, NULL);
	return FALSE;
}
