#include <dirent.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "bool.h"
#include "msg.h"
#include "str.h"



char *ent_from_path(char *path) {
	char *slash;

	if((slash = strrchr(path, '/')))
		return (slash + 1);
	return path;
}


BOOL is_dot(const char *file) {
	return (
		(file[0] == '.') &&
		(file[1] == 0 || (file[1] == '.' && file[2] == 0))
	);
}


int is_dir_v(const char *path) {
	struct stat st;

	if (lstat(path, &st) != 0) {
		err(MSG_STAT, path);
		return -1;
	}
	return S_ISDIR(st.st_mode);
}


int file_exist_v(const char *path) {
	struct stat st;

	if (stat(path, &st) == 0)
		return 1;
	if (errno == ENOENT)
		return 0;
	err(MSG_STAT, path);
	return -1;
}


DIR *opendir_v(const char *name) {
		DIR *dir = opendir(name);
		if (dir)
			return dir;
		msg(MSG_OPENDIR, strerror(errno), name);
		return dir;
}


BOOL get_inode(ino_t *inode, const char *path) {
	struct stat st;
	int ret = lstat(path, &st);

	*inode = st.st_ino;
	return ret == 0;
}


int safe_rename_v(const char *src, const char *dest) {
	int d = is_dir_v(src);

	if (d == -1)
		return -1;

	/* caller ensures that src != dest */
	if ((d ? mkdir(dest, S_IRWXU | S_IRWXG | S_IRWXO) : link(src, dest)) != 0) {
		if (errno == EEXIST)
			return 0;
		err(d ? MSG_MKDIR : MSG_LINK, dest);
		return -1;
	}
	if ((d ? rename(src, dest) : unlink(src)) == 0)
		return 1;
	err(MSG_REPLACE, dest);
	if ((d ? rmdir(dest) : unlink(dest)) != 0)
		err(MSG_REMOVE, dest);
	return 0;
}


void pretty_print_ent(const char *path) {
	/* is_dir_v() may print an error message, so we call it right away
	 * to avoid interleaving output of stderr and stdout. */
	BOOL slash = isatty(1) && (is_dir_v(path) == 1);

	WRITE(1, path);
	if (slash)
		WRITE(1, "/");
	WRITE(1, "\n");
}
